@@ -1,6 +1,7 @@
 //! High level program logic implementing the whole workflow.
 
 use crate::config::CFG;
+use crate::error::NoteError;
 use crate::error::WorkflowError;
 use crate::file_editor::launch_editor;
 use crate::filename;
@@ -139,39 +140,37 @@ fn create_new_note_or_synchronize_filename(path: &Path) -> Result<PathBuf, Workf
         {
             // CREATE A NEW NOTE BASED ON CLIPBOARD OR INPUT STREAM
             // (only if there is a valid YAML front matter)
-            log::trace!("Applying templates: `[tmpl] copy_content`, `[tmpl] copy_filename`");
-            let n = Note::from_content_template(path, &CFG.tmpl.copy_content).map_err(|e| {
-                WorkflowError::Template {
-                    tmpl_name: "[tmpl] copy_content".to_string(),
+            log::trace!("Applying templates: `[tmpl] from_clipboard_yaml_content`, `[tmpl] from_clipboard_yaml_filename`");
+            let n = Note::from_content_template(path, &CFG.tmpl.from_clipboard_yaml_content)
+                .map_err(|e| WorkflowError::Template {
+                    tmpl_name: "[tmpl] from_clipboard_yaml_content".to_string(),
                     source: e,
-                }
-            })?;
+                })?;
             // CREATE A NEW NOTE WITH `TMPL_COPY_CONTENT` TEMPLATE
-            let new_file_path = n.render_filename(&CFG.tmpl.copy_filename).map_err(|e| {
-                WorkflowError::Template {
-                    tmpl_name: "[tmpl] copy_filename".to_string(),
+            let new_file_path = n
+                .render_filename(&CFG.tmpl.from_clipboard_yaml_filename)
+                .map_err(|e| WorkflowError::Template {
+                    tmpl_name: "[tmpl] from_clipboard_yaml_filename".to_string(),
                     source: e,
-                }
-            })?;
+                })?;
             (n, new_file_path)
         } else {
             // CREATE A NEW NOTE BASED ON CLIPBOARD OR INPUT STREAM
             log::trace!(
-                "Applying templates: `[tmpl] clipboard_content`, `[tmpl] clipboard_filename`"
+                "Applying templates: `[tmpl] from_clipboard_content`, `[tmpl] from_clipboard_filename`"
             );
-            let n =
-                Note::from_content_template(path, &CFG.tmpl.clipboard_content).map_err(|e| {
-                    WorkflowError::Template {
-                        tmpl_name: "[tmpl] clipboard_content".to_string(),
-                        source: e,
-                    }
-                })?;
+            let n = Note::from_content_template(path, &CFG.tmpl.from_clipboard_content).map_err(
+                |e| WorkflowError::Template {
+                    tmpl_name: "[tmpl] from_clipboard_content".to_string(),
+                    source: e,
+                },
+            )?;
 
             // CREATE A NEW NOTE WITH `TMPL_CLIPBOARD_CONTENT` TEMPLATE
             let new_file_path = n
-                .render_filename(&CFG.tmpl.clipboard_filename)
+                .render_filename(&CFG.tmpl.from_clipboard_filename)
                 .map_err(|e| WorkflowError::Template {
-                    tmpl_name: "[tmpl] clipboard_filename".to_string(),
+                    tmpl_name: "[tmpl] from_clipboard_filename".to_string(),
                     source: e,
                 })?;
             (n, new_file_path)
@@ -185,9 +184,10 @@ fn create_new_note_or_synchronize_filename(path: &Path) -> Result<PathBuf, Workf
 
         Ok(new_file_path)
     } else {
+        // `path` points to a file.
+
         let extension_is_known = !matches!(
             MarkupLanguage::from(
-                None,
                 path.extension()
                     .unwrap_or_default()
                     .to_str()
@@ -199,8 +199,41 @@ fn create_new_note_or_synchronize_filename(path: &Path) -> Result<PathBuf, Workf
         if extension_is_known {
             // SYNCHRONIZE FILENAME
             // `path` points to an existing Tp-Note file.
-            // Check if in sync with its filename:
-            Ok(synchronize_filename(path)?)
+            // Check if in sync with its filename.
+            // If the note file has no header, we prepend one if wished for.
+            match (
+                synchronize_filename(path),
+                // Shall we prepend a header, in case it is missing?
+                (ARGS.add_header || CFG.arg_default.add_header)
+                    && !CFG.arg_default.no_filename_sync
+                    && !ARGS.no_filename_sync,
+            ) {
+                (Ok(path), _) => Ok(path),
+                (
+                    Err(WorkflowError::Note {
+                        source: NoteError::MissingFrontMatter { .. },
+                    }),
+                    true,
+                ) => {
+                    log::trace!(
+                       "Applying template: `[tmpl] from_text_file_content`, `[tmpl] from_text_file_filename`"
+                    );
+                    let n = Note::from_text_file(path, &CFG.tmpl.from_text_file_content)?;
+                    let new_file_path = n
+                        .render_filename(&CFG.tmpl.from_text_file_filename)
+                        .map_err(|e| WorkflowError::Template {
+                            tmpl_name: "[tmpl] from_text_file_filename".to_string(),
+                            source: e,
+                        })?;
+                    n.content.write_to_disk(&*new_file_path)?;
+                    if path != new_file_path {
+                        log::trace!("Deleting file: {:?}", path);
+                        fs::remove_file(path)?;
+                    }
+                    Ok(new_file_path)
+                }
+                (Err(e), _) => Err(e),
+            }
         } else {
             // Error if we are supposed to export an unknown file type.
             if ARGS.export.is_some() {
@@ -210,19 +243,19 @@ fn create_new_note_or_synchronize_filename(path: &Path) -> Result<PathBuf, Workf
             // ANNOTATE FILE: CREATE NEW NOTE WITH TMPL_ANNOTATE_CONTENT TEMPLATE
             // `path` points to a foreign file type that will be annotated.
             log::trace!(
-                "Applying templates `[tmpl] annotate_content` and `[tmpl] annotate_filename`."
+                "Applying templates `[tmpl] annotate_file_content` and `[tmpl] annotate_file_filename`."
             );
-            let n = Note::from_content_template(path, &CFG.tmpl.annotate_content).map_err(|e| {
-                WorkflowError::Template {
-                    tmpl_name: "[tmpl] annotate_content".to_string(),
+            let n = Note::from_content_template(path, &CFG.tmpl.annotate_file_content).map_err(
+                |e| WorkflowError::Template {
+                    tmpl_name: "[tmpl] annotate_file_content".to_string(),
                     source: e,
-                }
-            })?;
+                },
+            )?;
 
             let new_file_path = n
-                .render_filename(&CFG.tmpl.annotate_filename)
+                .render_filename(&CFG.tmpl.annotate_file_filename)
                 .map_err(|e| WorkflowError::Template {
-                    tmpl_name: "[tmpl] annotate_filename".to_string(),
+                    tmpl_name: "[tmpl] annotate_file_filename".to_string(),
                     source: e,
                 })?;
 
@@ -265,20 +298,20 @@ pub fn run() -> Result<PathBuf, WorkflowError> {
                 launch_viewer = *LAUNCH_VIEWER;
             }
         }
-        Err(e) => {
-            if (matches!(e, WorkflowError::InvalidFrontMatterYaml { .. })
-                || matches!(e, WorkflowError::MissingFrontMatter { .. })
-                || matches!(e, WorkflowError::MissingFrontMatterField { .. })
-                || matches!(e, WorkflowError::CompulsoryFrontMatterFieldIsEmpty { .. })
-                || matches!(e, WorkflowError::SortTagVarInvalidChar { .. })
-                || matches!(e, WorkflowError::FileExtNotRegistered { .. }))
+        Err(WorkflowError::Note { source: e }) => {
+            if (matches!(e, NoteError::InvalidFrontMatterYaml { .. })
+                || matches!(e, NoteError::MissingFrontMatter { .. })
+                || matches!(e, NoteError::MissingFrontMatterField { .. })
+                || matches!(e, NoteError::CompulsoryFrontMatterFieldIsEmpty { .. })
+                || matches!(e, NoteError::SortTagVarInvalidChar { .. })
+                || matches!(e, NoteError::FileExtNotRegistered { .. }))
                 && !ARGS.batch
                 && ARGS.export.is_none()
             {
                 // Continue the workflow.
 
-                let missing_header = matches!(e, WorkflowError::MissingFrontMatter { .. })
-                    || matches!(e, WorkflowError::MissingFrontMatterField { .. });
+                let missing_header = matches!(e, NoteError::MissingFrontMatter { .. })
+                    || matches!(e, NoteError::MissingFrontMatterField { .. });
 
                 launch_viewer = *LAUNCH_VIEWER
                     && !(missing_header && CFG.viewer.missing_header_disables && !ARGS.view);
@@ -292,9 +325,11 @@ pub fn run() -> Result<PathBuf, WorkflowError> {
                 };
             } else {
                 // This is a fatal error, so we quit.
-                return Err(e);
+                return Err(WorkflowError::Note { source: e });
             }
         }
+        // This is a fatal error, so we quit.
+        Err(e) => return Err(e),
     };
 
     #[cfg(feature = "viewer")]
@@ -319,8 +354,17 @@ pub fn run() -> Result<PathBuf, WorkflowError> {
             // `path` has changed!
             Ok(p) => path = p,
             Err(e) => {
-                let missing_header = matches!(e, WorkflowError::MissingFrontMatter { .. })
-                    || matches!(e, WorkflowError::MissingFrontMatterField { .. });
+                let missing_header = matches!(
+                    e,
+                    WorkflowError::Note {
+                        source: NoteError::MissingFrontMatter { .. }
+                    }
+                ) || matches!(
+                    e,
+                    WorkflowError::Note {
+                        source: NoteError::MissingFrontMatterField { .. }
+                    }
+                );
 
                 if missing_header {
                     // Silently ignore error.
